# Generated by Django 2.0.1 on 2018-01-22 22:15

from django.db import migrations


class Migration(migrations.Migration):

    def create_administrators(apps, schema_editor):
        User = apps.get_model("person", "User")
        User.objects.filter(is_superuser=True).update(is_superuser=False, is_administrator=True)

    dependencies = [
        ('person', '0012_user_is_administrator'),
    ]

    operations = [
        migrations.RunPython(create_administrators, migrations.RunPython.noop),
    ]



from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from zxcvbn import zxcvbn


class ZXCVBNValidator:
    code = 'password_too_weak'
    DEFAULT_USER_ATTRIBUTES = ('username', 'first_name', 'last_name', 'email')

    def __init__(self, min_score=3, user_attributes=DEFAULT_USER_ATTRIBUTES):
        if not (0 <= min_score <= 4):
            raise Exception('min_score must be between 0 and 4!')
        self.min_score = min_score
        self.user_attributes = user_attributes

    def __call__(self, value):
        return self.validate(value)

    def validate(self, password, user=None):
        user_inputs = [getattr(user, attribute, None) for attribute in self.user_attributes]
        user_inputs = [attr for attr in user_inputs if attr is not None]
        results = zxcvbn(password, user_inputs=user_inputs)
        if results.get('score', 0) < self.min_score:
            feedback = ', '.join(results.get('feedback', {}).get('suggestions', []))
            raise ValidationError(_(feedback), code=self.code, params={})

    def get_help_text(self):
        return _('Your password is too weak.')



import urllib

from django.contrib import messages
from django.db.models import Q
from django.http import Http404, JsonResponse
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.http import is_safe_url
from django.utils.translation import ugettext as _
from django.views.generic import View

from pretalx.person.models import EventPermission, User


class UserList(View):

    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perm('orga.search_all_users', request.event):
            raise Http404()

        search = request.GET.get('search')
        if not search or len(search) < 2:
            return JsonResponse({'count': 0, 'results': []})

        queryset = User.objects.filter(
            Q(nick__icontains=search) | Q(name__icontains=search)
        )
        if request.GET.get('orga', 'false').lower() == 'true':
            permissions = EventPermission.objects.filter(event=self.request.event, is_orga=True)
            queryset = queryset.filter(permissions__in=permissions)

        return JsonResponse({
            'count': len(queryset),
            'results': [
                {
                    'nick': user.nick,
                    'name': user.name,
                }
                for user in queryset
            ],
        })


class SubuserView(View):

    def dispatch(self, request, *args, **kwargs):
        request.user.is_administrator = request.user.is_superuser
        request.user.is_superuser = False
        request.user.save(update_fields=['is_administrator', 'is_superuser'])
        messages.success(request, _('You are now an administrator instead of a superuser.'))
        params = request.GET.copy()
        url = urllib.parse.unquote(params.pop('next', [''])[0])
        if url and is_safe_url(url, request.get_host()):
            return redirect(url + ('?' + params.urlencode() if params else ''))
        return redirect(reverse('orga:dashboard'))



from django import forms
from django.utils.translation import ugettext_lazy as _

from pretalx.common.mixins.forms import ReadOnlyFlag
from pretalx.common.phrases import phrases
from pretalx.submission.models import Review


class ReviewForm(ReadOnlyFlag, forms.ModelForm):

    def __init__(self, event, user, *args, instance=None, **kwargs):
        self.event = event
        self.may_override = event.settings.allow_override_votes and user.remaining_override_votes(event)
        self.may_override = self.may_override or (instance and instance.override_vote is not None)
        self.min_value = int(event.settings.review_min_score)
        self.max_value = int(event.settings.review_max_score)
        if instance:
            if instance.override_vote is True:
                instance.score = self.max_value + 1
            elif instance.override_vote is False:
                instance.score = self.min_value - 1

        super().__init__(*args, instance=instance, **kwargs)
        choices = [(None, _('No score'))]
        if self.may_override:
            choices.append((self.min_value - 1, _('Negative override (Veto)')))
        for counter in range(abs(self.max_value - self.min_value) + 1):
            value = self.min_value + counter
            name = event.settings.get(f'review_score_name_{value}')
            if name:
                name = f'{value} (»{name}«)'
            else:
                name = value
            choices.append((value, name))
        if self.may_override:
            choices.append((self.max_value + 1, _('Positive override')))

        self.fields['score'] = forms.ChoiceField(choices=choices, required=False, disabled=kwargs.get('read_only', False))
        self.fields['text'].widget.attrs['rows'] = 2
        self.fields['text'].widget.attrs['placeholder'] = phrases.orga.example_review

    def clean_score(self):
        score = self.cleaned_data.get('score')
        score = int(score) if score else None
        if score and not self.min_value <= score <= self.max_value:
            if not ((score == self.min_value - 1 or score == self.max_value + 1) and self.may_override):
                raise forms.ValidationError(_(f'Please assign a score between {self.min_value} and {self.max_value}!'))
        return score

    def clean(self):
        cleaned_data = super().clean()
        score = cleaned_data.get('score')
        if score == self.min_value - 1:
            cleaned_data['score'] = None
            if self.may_override:
                self.instance.override_vote = False
        elif score == self.max_value + 1:
            cleaned_data['score'] = None
            if self.may_override:
                self.instance.override_vote = True
        else:
            self.instance.override_vote = None
        if self.instance.id:
            self.instance.save()

    class Meta:
        model = Review
        fields = (
            'text', 'score'
        )



# Generated by Django 2.0.1 on 2018-02-02 17:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('submission', '0018_remove_submissiontype_max_duration'),
    ]

    operations = [
        migrations.AddField(
            model_name='submissiontype',
            name='deadline',
            field=models.DateTimeField(blank=True, help_text='If you want a different deadline than the global deadline for this submission type, enter it here.', null=True, verbose_name='deadline'),
        ),
    ]



import urllib

from django.contrib import messages
from django.contrib.auth import authenticate, login, logout
from django.http import HttpRequest, HttpResponseRedirect
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.http import is_safe_url
from django.utils.translation import ugettext as _
from django.views.generic import TemplateView

from pretalx.common.phrases import phrases


class LoginView(TemplateView):
    template_name = 'orga/auth/login.html'

    def post(self, request: HttpRequest, *args, **kwargs) -> HttpResponseRedirect:
        username = request.POST.get('username')
        password = request.POST.get('password')
        user = authenticate(username=username, password=password)

        if user is None:
            messages.error(request, _('No user account matches the entered credentials.'))
            return redirect('orga:login')

        if not user.is_active:
            messages.error(request, _('User account is deactivated.'))
            return redirect('orga:login')

        login(request, user)
        params = request.GET.copy()
        url = urllib.parse.unquote(params.pop('next', [''])[0])
        if url and is_safe_url(url, request.get_host()):
            return redirect(url + ('?' + params.urlencode() if params else ''))

        # check where to reasonably redirect:
        # orga of a running event? go to that event.
        messages.success(request, phrases.orga.logged_in)
        if request.user.permissions.count() == 1:
            return redirect(reverse('orga:event.dashboard', kwargs={'event': request.user.permissions.first().event.slug}))
        return redirect(reverse('orga:dashboard'))


def logout_view(request: HttpRequest) -> HttpResponseRedirect:
    logout(request)
    return redirect(reverse('orga:login'))



# Generated by Django 2.0.1 on 2018-02-02 17:16

from django.db import migrations


def fix_log_action(apps, schema_editor):
    ActivityLog = apps.get_model('common', 'ActivityLog')
    ActivityLog.objects.all().filter(action_type='update').update(action_type='pretalx.submission.update')


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0004_auto_20170526_0437'),
    ]

    operations = [
        migrations.RunPython(fix_log_action, migrations.RunPython.noop)
    ]



from django.forms import PasswordInput
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _


class PasswordStrengthInput(PasswordInput):

    def render(self, name, value, attrs=None):
        markup = """
        <div class="password-progress">
            <div class="password-progress-bar progress">
                <div class="progress-bar progress-bar-warning password_strength_bar"
                     role="progressbar"
                     aria-valuenow="0"
                     aria-valuemin="0"
                     aria-valuemax="4">
                </div>
            </div>
            <p class="text-muted password_strength_info hidden">
                <span style="margin-left:5px;">
                    {message}
                </span>
            </p>
        </div>
        """.format(message=_('This password would take <em class="password_strength_time"></em> to crack.'))

        self.attrs['class'] = ' '.join(self.attrs.get('class', '').split(' ') + ['password_strength'])
        return mark_safe(super().render(name, value, self.attrs) + markup)

    class Media(object):
        js = ('common/js/zxcvbn.js', 'common/js/password_strength.js')


class PasswordConfirmationInput(PasswordInput):

    def __init__(self, confirm_with=None, attrs=None, render_value=False):
        super().__init__(attrs, render_value)
        self.confirm_with = confirm_with

    def render(self, name, value, attrs=None):
        if self.confirm_with:
            self.attrs['data-confirm-with'] = f'{self.confirm_with}'

        markup = """
        <div class="hidden password_strength_info">
            <p class="text-muted">
                <span class="label label-danger">{warning}</span>
                <span>{content}</span>
            </p>
        </div>
        """.format(warning=_('Warning'), content=_("Your passwords don't match."))

        self.attrs['class'] = ' '.join(self.attrs.get('class', '').split(' ') + ['password_confirmation'])

        return mark_safe(super(PasswordInput, self).render(name, value, self.attrs) + markup)



from django import forms
from django.conf import settings
from django.utils.timezone import now

from pretalx.submission.models import Submission, SubmissionType


class InfoForm(forms.ModelForm):

    def __init__(self, event, **kwargs):
        self.event = event
        readonly = kwargs.pop('readonly', False)

        super().__init__(**kwargs)
        instance = kwargs.get('instance')
        self.fields['submission_type'].queryset = SubmissionType.objects.filter(event=self.event)
        self.initial['submission_type'] = getattr(instance, 'submission_type', self.event.cfp.default_type)
        _now = now()
        if not self.event.cfp.deadline or self.event.cfp.deadline >= _now:  # No global deadline or still open
            types = self.event.submission_types.exclude(deadline__lt=_now)
        else:
            types = self.event.submission_types.filter(deadline__gte=_now)
        pks = set(types.values_list('pk', flat=True))
        if instance and instance.pk:
            pks |= {instance.submission_type.pk, }

        self.fields['submission_type'].queryset = self.event.submission_types.filter(pk__in=pks)

        locale_names = dict(settings.LANGUAGES)
        self.fields['content_locale'].choices = [(a, locale_names[a]) for a in self.event.locales]
        if readonly:
            for f in self.fields.values():
                f.disabled = True

    class Meta:
        model = Submission
        fields = [
            'title', 'submission_type', 'content_locale', 'abstract',
            'description', 'notes', 'do_not_record',
        ]



from django.forms import FileField, Form, ModelForm

from pretalx.schedule.models import Schedule


class ScheduleReleaseForm(ModelForm):

    class Meta:
        model = Schedule
        fields = ('version', )


class ScheduleImportForm(Form):
    upload = FileField()

    class Meta:
        fields = ('upload', )



import xml.etree.ElementTree as ET
from datetime import datetime

from django.core.management.base import BaseCommand
from django.db import transaction

from pretalx.event.models import Event
from pretalx.person.models import EventPermission, User


class Command(BaseCommand):
    help = 'Imports a frab xml export'

    def add_arguments(self, parser):
        parser.add_argument('path', type=str)

    @transaction.atomic
    def handle(self, *args, **options):
        from pretalx.schedule.utils import process_frab
        path = options.get('path')
        tree = ET.parse(path)
        root = tree.getroot()

        event_data = root.find('conference')
        event = Event.objects.filter(slug__iexact=event_data.find('acronym').text).first()
        if not event:
            event = Event(
                name=event_data.find('title').text,
                slug=event_data.find('acronym').text,
                date_from=datetime.strptime(event_data.find('start').text, '%Y-%m-%d').date(),
                date_to=datetime.strptime(event_data.find('end').text, '%Y-%m-%d').date(),
            )
            event.save()

        for user in User.objects.filter(is_administrator=True):
            EventPermission.objects.get_or_create(event=event, user=user, is_orga=True)

        self.stdout.write(self.style.SUCCESS(process_frab(root, event)))



from rest_framework.serializers import ModelSerializer, SlugRelatedField

from pretalx.person.models import User
from pretalx.submission.models import Answer, AnswerOption, Question, Submission


class AnswerOptionSerializer(ModelSerializer):

    class Meta:
        model = AnswerOption
        fields = ('id', 'answer')


class QuestionSerializer(ModelSerializer):
    options = AnswerOptionSerializer(many=True)

    class Meta:
        model = Question
        fields = ('id', 'question', 'required', 'target', 'options')


class AnswerSerializer(ModelSerializer):
    question = QuestionSerializer(Question.objects.all(), read_only=True)
    submission = SlugRelatedField(queryset=Submission.objects.all(), slug_field='code')
    person = SlugRelatedField(queryset=User.objects.all(), slug_field='code')
    options = AnswerOptionSerializer(many=True)

    class Meta:
        model = Answer
        fields = ('id', 'question', 'answer', 'answer_file', 'submission', 'person', 'options')



from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "Rebuild static files and language files"

    def add_arguments(self, parser):
        parser.add_argument(
            '-c', '--clear', action='store_true', dest='clear',
            help="Clear the existing files using the storage "
                 "before trying to copy or link the original file.",
        )

    def handle(self, *args, **options):
        call_command('compilemessages', verbosity=1)
        call_command('collectstatic', verbosity=1, interactive=False, clear=options['clear'])
        call_command('compress', verbosity=1)



from rest_framework import viewsets

from pretalx.api.serializers.submission import (
    ScheduleListSerializer, ScheduleSerializer, SubmissionSerializer,
)
from pretalx.schedule.models import Schedule
from pretalx.submission.models import Submission


class SubmissionViewSet(viewsets.ReadOnlyModelViewSet):
    serializer_class = SubmissionSerializer
    queryset = Submission.objects.none()
    lookup_field = 'code__iexact'
    filter_fields = ('state', 'content_locale', 'submission_type')
    search_fields = ('title', 'speakers__name')

    def get_base_queryset(self):
        if self.request.user.has_perm('orga.view_submissions', self.request.event):
            return self.request.event.submissions.all()
        if self.request.event.current_schedule and self.request.event.settings.show_schedule:
            return self.request.event.submissions.filter(slots__in=self.request.event.current_schedule.talks.all())

    def get_queryset(self):
        qs = self.get_base_queryset() or self.queryset
        if 'talks' in self.request._request.path:
            qs = qs.filter(slots__schedule=self.request.event.current_schedule)
        return qs


class ScheduleViewSet(viewsets.ReadOnlyModelViewSet):
    serializer_class = ScheduleSerializer
    queryset = Schedule.objects.none()
    lookup_field = 'version__iexact'

    def get_serializer_class(self):
        if self.action == 'retrieve':
            return ScheduleSerializer
        elif self.action == 'list':
            return ScheduleListSerializer

    def get_object(self):
        try:
            return super().get_object()
        except Exception as e:
            is_public = self.request.event.is_public and self.request.event.settings.show_schedule
            has_perm = self.request.user.has_perm('orga.edit_schedule', self.request.event)
            query = self.kwargs.get(self.lookup_field)
            if has_perm and query == 'wip':
                return self.request.event.wip_schedule
            elif (has_perm or is_public) and query == 'latest' and self.request.event.current_schedule:
                return self.request.event.current_schedule
            raise

    def get_queryset(self):
        qs = self.queryset
        is_public = self.request.event.is_public and self.request.event.settings.show_schedule
        current_schedule = self.request.event.current_schedule.pk if self.request.event.current_schedule else None

        if self.request.user.has_perm('orga.view_schedule', self.request.event):
            return self.request.event.schedules.all()
        if is_public:
            return self.request.event.schedules.filter(pk=current_schedule)
        return qs



# Generated by Django 2.0.1 on 2018-01-26 15:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('submission', '0017_auto_20180115_1743'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='submissiontype',
            name='max_duration',
        ),
    ]



import json
import os.path
import xml.etree.ElementTree as ET
from datetime import timedelta

import dateutil.parser
from django.contrib import messages
from django.db import transaction
from django.db.models.deletion import ProtectedError
from django.http import FileResponse, JsonResponse
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import FormView, TemplateView, View
from i18nfield.utils import I18nJSONEncoder

from pretalx.agenda.management.commands.export_schedule_html import (
    Command as ExportScheduleHtml,
)
from pretalx.agenda.tasks import export_schedule_html
from pretalx.common.mixins.views import ActionFromUrl, PermissionRequired
from pretalx.common.views import CreateOrUpdateView
from pretalx.orga.forms.schedule import ScheduleImportForm, ScheduleReleaseForm
from pretalx.orga.views.event import EventSettingsPermission
from pretalx.schedule.forms import RoomForm
from pretalx.schedule.models import Availability, Room


class ScheduleView(PermissionRequired, TemplateView):
    template_name = 'orga/schedule/index.html'
    permission_required = 'orga.view_schedule'

    def get_permission_object(self):
        return self.request.event

    def get_context_data(self, event):
        ctx = super().get_context_data()
        version = self.request.GET.get('version')
        ctx['schedule_version'] = version
        ctx['active_schedule'] = self.request.event.schedules.get(version=version) if version else self.request.event.wip_schedule
        ctx['release_form'] = ScheduleReleaseForm()
        return ctx


class ScheduleExportView(PermissionRequired, TemplateView):
    template_name = 'orga/schedule/export.html'
    permission_required = 'orga.view_schedule'

    def get_permission_object(self):
        return self.request.event


class ScheduleExportTriggerView(PermissionRequired, View):
    permission_required = 'orga.view_schedule'

    def get_permission_object(self):
        return self.request.event

    def post(self, request, event):
        export_schedule_html.apply_async(kwargs={'event_id': self.request.event.id})
        messages.success(self.request, _('A new export is being generated and will be available soon.'))
        return redirect(self.request.event.orga_urls.schedule_export)


class ScheduleExportDownloadView(PermissionRequired, View):
    permission_required = 'orga.view_schedule'

    def get_permission_object(self):
        return self.request.event

    def get(self, request, event):
        zip_path = ExportScheduleHtml.get_output_zip_path(self.request.event)
        zip_name = os.path.basename(zip_path)
        response = FileResponse(open(zip_path, 'rb'), content_type='application/force-download')
        response['Content-Disposition'] = 'attachment; filename=' + zip_name
        return response


class ScheduleReleaseView(PermissionRequired, View):
    form_class = ScheduleReleaseForm
    permission_required = 'orga.release_schedule'

    def get_permission_object(self):
        return self.request.event

    def post(self, request, event):
        form = ScheduleReleaseForm(self.request.POST)
        form.is_valid()
        if self.request.event.schedules.filter(version=form.cleaned_data['version']).exists():
            messages.error(self.request, _('Please use a version number you did not use yet!'))
        else:
            self.request.event.release_schedule(form.cleaned_data['version'], user=request.user)
            messages.success(self.request, _('Nice, your schedule has been released!'))
        return redirect(self.request.event.orga_urls.schedule)


class ScheduleResetView(PermissionRequired, View):
    permission_required = 'orga.edit_schedule'

    def get_permission_object(self):
        return self.request.event

    def dispatch(self, request, event):
        schedule_version = self.request.GET.get('version')
        self.request.event.schedules.get(version=schedule_version).unfreeze(user=request.user)
        messages.success(self.request, _('Reset successful – start editing the schedule from your selcted version!'))
        return redirect(self.request.event.orga_urls.schedule)


class ScheduleToggleView(PermissionRequired, View):
    permission_required = 'orga.edit_schedule'

    def get_permission_object(self):
        return self.request.event

    def dispatch(self, request, event):
        self.request.event.settings.set('show_schedule', not self.request.event.settings.show_schedule)
        return redirect(self.request.event.orga_urls.schedule)


class RoomListApi(PermissionRequired, View):
    permission_required = 'orga.view_room'

    def get_permission_object(self):
        return self.request.event

    def get(self, request, event):
        return JsonResponse({
            'start': request.event.datetime_from.isoformat(),
            'end': request.event.datetime_to.isoformat(),
            'timezone': request.event.timezone,
            'rooms': [
                {
                    'id': room.pk,
                    'name': str(room.name),
                    'description': room.description,
                    'capacity': room.capacity,
                    'url': room.urls.edit,
                    'availabilities': [avail.serialize() for avail in room.availabilities.all()]
                }
                for room in request.event.rooms.order_by('position')
            ]
        }, encoder=I18nJSONEncoder)


def serialize_slot(slot):
    return {
        'id': slot.pk,
        'title': str(slot.submission.title),
        'speakers': [
            {'name': speaker.name, 'nick': speaker.nick}
            for speaker in slot.submission.speakers.all()
        ],
        'submission_type': str(slot.submission.submission_type.name),
        'state': slot.submission.state,
        'description': str(slot.submission.description),
        'abstract': str(slot.submission.abstract),
        'notes': slot.submission.notes,
        'duration': slot.submission.duration or slot.submission.submission_type.default_duration,
        'content_locale': slot.submission.content_locale,
        'do_not_record': slot.submission.do_not_record,
        'room': slot.room.pk if slot.room else None,
        'start': slot.start.isoformat() if slot.start else None,
        'end': slot.end.isoformat() if slot.end else None,
        'url': slot.submission.orga_urls.base,
    }


class TalkList(PermissionRequired, View):
    permission_required = 'orga.edit_schedule'

    def get_permission_object(self):
        return self.request.event

    def get(self, request, event):
        version = self.request.GET.get('version')
        if version:
            schedule = request.event.schedules.get(version=version)
        else:
            schedule = request.event.wip_schedule

        return JsonResponse(
            {'results': [serialize_slot(slot) for slot in schedule.talks.all()]},
            encoder=I18nJSONEncoder
        )


class TalkUpdate(PermissionRequired, View):
    permission_required = 'orga.schedule_talk'

    def get_object(self):
        return self.request.event.wip_schedule.talks.filter(pk=self.kwargs.get('pk')).first()

    def patch(self, request, event, pk):
        talk = self.get_object()
        data = json.loads(request.body.decode())

        if data.get('start'):
            talk.start = dateutil.parser.parse(data.get('start'))
            talk.end = talk.start + timedelta(minutes=talk.submission.get_duration())
        else:
            talk.start = None
            talk.end = None

        if data.get('room'):
            talk.room = request.event.rooms.get(pk=data.get('room'))
        else:
            talk.room = None

        talk.save(update_fields=['start', 'end', 'room'])

        return JsonResponse(serialize_slot(talk))


class RoomTalkAvailabilities(PermissionRequired, View):
    permission_required = 'orga.edit_room'

    def get_permission_object(self):
        return self.request.event

    def get(self, request, event, talkid, roomid):
        talk = request.event.wip_schedule.talks.get(pk=talkid)
        room = request.event.rooms.get(pk=roomid)

        availabilitysets = [
            room.availabilities.all(),
            *[
                speaker.profiles.filter(event=request.event).first().availabilities.all()
                for speaker in talk.submission.speakers.all()
                if speaker.profiles.filter(event=request.event).exists()
            ],
        ]

        availabilities = Availability.intersection(*availabilitysets)

        return JsonResponse({
            'results': [
                avail.serialize() for avail in availabilities
            ],
        })


class ScheduleImportView(PermissionRequired, FormView):
    permission_required = 'orga.release_schedule'
    template_name = 'orga/schedule/import.html'
    form_class = ScheduleImportForm

    def get_permission_object(self):
        return self.request.event

    def get_success_url(self):
        return self.request.event.orga_urls.schedule_import

    def form_valid(self, form):
        from pretalx.schedule.utils import process_frab
        try:
            tree = ET.parse('path')
            root = tree.getroot()
        except Exception as e:
            messages.error(self.request, _('Unable to parse XML file: ') + str(e))
        try:
            with transaction.atomic():
                messages.success(self.request, process_frab(root, self.request.event))
            return super().form_valid(form)
        except Exception as e:
            messages.error(self.request, _('Unable to release new schedule: ' + str(e)))
        return super().form_invalid(form)


class RoomList(EventSettingsPermission, TemplateView):
    template_name = 'orga/schedule/room_list.html'


class RoomDelete(EventSettingsPermission, View):
    permission_required = 'orga.edit_room'

    def dispatch(self, request, event, pk):
        try:
            request.event.rooms.get(pk=pk).delete()
            messages.success(self.request, _('Room deleted. Hopefully nobody was still in there …'))
        except ProtectedError:  # TODO: show which/how many talks are concerned
            messages.error(request, _('There is or was a talk scheduled in this room. It cannot be deleted.'))

        return redirect(request.event.orga_urls.room_settings)


class RoomDetail(EventSettingsPermission, ActionFromUrl, CreateOrUpdateView):
    model = Room
    form_class = RoomForm
    template_name = 'orga/schedule/room_form.html'
    permission_required = 'orga.view_room'

    @property
    def write_permission_required(self):
        if 'pk' not in self.kwargs:
            return 'orga.change_settings'
        return 'orga.edit_room'

    def get_object(self):
        try:
            return self.request.event.rooms.get(pk=self.kwargs['pk'])
        except (Room.DoesNotExist, KeyError):
            return

    def get_success_url(self) -> str:
        return self.request.event.orga_urls.room_settings

    def get_form_kwargs(self, *args, **kwargs):
        kwargs = super().get_form_kwargs(*args, **kwargs)
        kwargs['event'] = self.request.event
        return kwargs

    def form_valid(self, form):
        form.instance.event = self.request.event
        created = not bool(form.instance.pk)
        ret = super().form_valid(form)
        messages.success(self.request, _('Saved!'))
        if created:
            form.instance.log_action('pretalx.room.create', person=self.request.user, orga=True)
        else:
            form.instance.log_action('pretalx.event.update', person=self.request.user, orga=True)
        return ret



from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _
from django.views.generic import TemplateView

from pretalx.common.mixins.views import PermissionRequired
from pretalx.common.models.log import ActivityLog
from pretalx.event.stages import get_stages


class DashboardView(TemplateView):
    template_name = 'orga/dashboard.html'


class EventDashboardView(PermissionRequired, TemplateView):
    template_name = 'orga/event/dashboard.html'
    permission_required = 'orga.view_orga_area'

    def get_object(self):
        return self.request.event

    def get_context_data(self, event):
        ctx = super().get_context_data()
        event = self.request.event
        ctx['timeline'] = get_stages(event)
        ctx['history'] = ActivityLog.objects.filter(event=self.get_object())[:20]
        ctx['tiles'] = []
        today = now().date()
        if today < event.date_from:
            ctx['tiles'].append({
                'large': (event.date_from - today).days,
                'small': _('days until event start'),
            })
        elif today > event.date_to:
            ctx['tiles'].append({
                'large': (today - event.date_from).days,
                'small': _('days since event end'),
            })
        else:
            day = (today - event.date_from).days + 1
            ctx['tiles'].append({
                'large': _('Day {number}').format(number=day),
                'small': _('of {total_days} days').format(total_days=(event.date_to - event.date_from).days + 1),
                'url': event.urls.schedule + f'#{today.isoformat()}',
            })
        if event.current_schedule:
            ctx['tiles'].append({
                'large': event.current_schedule.version,
                'small': _('current schedule'),
                'url': event.urls.schedule,
            })
        ctx['tiles'].append({
            'url': event.urls.base,
            'small': _('Go to CfP'),
        })
        if event.submissions.count():
            ctx['tiles'].append({
                'large': event.submissions.count(),
                'small': _('total submissions'),
                'url': event.orga_urls.submissions,
            })
            if event.talks.count():
                ctx['tiles'].append({
                    'large': event.talks.count(),
                    'small': _('total talks'),
                    'url': event.orga_urls.submissions,
                })
        if event.speakers.count():
            ctx['tiles'].append({
                'large': event.speakers.count(),
                'small': _('speakers'),
                'url': event.orga_urls.speakers,
            })
        ctx['tiles'].append({
            'large': event.queued_mails.filter(sent__isnull=False).count(),
            'small': _('sent emails'),
            'url': event.orga_urls.send_mails,
        })
        return ctx



from decimal import Decimal

from django import forms
from django.core.files.uploadedfile import UploadedFile
from django.utils.functional import cached_property

from pretalx.submission.models import Answer, QuestionVariant


class QuestionsForm(forms.Form):

    def __init__(self, *args, **kwargs):
        self.event = kwargs.pop('event')
        self.submission = kwargs.pop('submission', None)
        self.speaker = kwargs.pop('speaker', None)
        self.request_user = kwargs.pop('request_user', None)
        self.target_type = kwargs.pop('target', 'submission')
        target_object = self.submission or self.speaker if self.target_type else None
        readonly = kwargs.pop('readonly', False)

        super().__init__(*args, **kwargs)

        queryset = self.event.questions.all()
        if self.target_type:
            queryset = queryset.filter(target=self.target_type)
        for question in queryset.prefetch_related('options'):
            if target_object:
                answers = [a for a in target_object.answers.all() if a.question_id == question.id]
                if answers:
                    initial_object = answers[0]
                    initial = answers[0].answer_file if question.variant == QuestionVariant.FILE else answers[0].answer
                else:
                    initial_object = None
                    initial = question.default_answer
            else:
                initial_object = None
                initial = question.default_answer

            field = self.get_field(question=question, initial=initial, initial_object=initial_object, readonly=readonly)
            field.question = question
            field.answer = initial_object
            self.fields[f'question_{question.pk}'] = field

    @cached_property
    def speaker_fields(self):
        return [forms.BoundField(self, field, name) for name, field in self.fields.items() if field.question.target == 'speaker']

    @cached_property
    def submission_fields(self):
        return [forms.BoundField(self, field, name) for name, field in self.fields.items() if field.question.target == 'submission']

    def get_field(self, *, question, initial, initial_object, readonly):
        if question.variant == QuestionVariant.BOOLEAN:
            # For some reason, django-bootstrap4 does not set the required attribute
            # itself.
            widget = forms.CheckboxInput(attrs={'required': 'required'}) if question.required else forms.CheckboxInput()
            initialbool = (initial == 'True') if initial else bool(question.default_answer)

            return forms.BooleanField(
                disabled=readonly, help_text=question.help_text,
                label=question.question, required=question.required,
                widget=widget, initial=initialbool
            )
        elif question.variant == QuestionVariant.NUMBER:
            return forms.DecimalField(
                disabled=readonly, help_text=question.help_text,
                label=question.question, required=question.required,
                min_value=Decimal('0.00'), initial=initial
            )
        elif question.variant == QuestionVariant.STRING:
            return forms.CharField(
                disabled=readonly, help_text=question.help_text,
                label=question.question, required=question.required, initial=initial
            )
        elif question.variant == QuestionVariant.TEXT:
            return forms.CharField(
                label=question.question, required=question.required,
                widget=forms.Textarea,
                disabled=readonly, help_text=question.help_text,
                initial=initial
            )
        elif question.variant == QuestionVariant.FILE:
            return forms.FileField(
                label=question.question, required=question.required,
                disabled=readonly, help_text=question.help_text,
                initial=initial
            )
        elif question.variant == QuestionVariant.CHOICES:
            return forms.ModelChoiceField(
                queryset=question.options.all(),
                label=question.question, required=question.required,
                initial=initial_object.options.first() if initial_object else question.default_answer,
                disabled=readonly, help_text=question.help_text,
            )
        elif question.variant == QuestionVariant.MULTIPLE:
            return forms.ModelMultipleChoiceField(
                queryset=question.options.all(),
                label=question.question, required=question.required,
                widget=forms.CheckboxSelectMultiple,
                initial=initial_object.options.all() if initial_object else question.default_answer,
                disabled=readonly, help_text=question.help_text,
            )

    def save(self):
        for k, v in self.cleaned_data.items():
            field = self.fields[k]
            if field.answer:
                # We already have a cached answer object, so we don't
                # have to create a new one
                if v == '':
                    # TODO: Deleting the answer removes the option to have a log here.
                    # Maybe setting the answer to '' is the right way to go.
                    field.answer.delete()
                else:
                    self._save_to_answer(field, field.answer, v)
                    field.answer.save()
            elif v != '':
                # Not distinguishing between the question types helps to make
                # experiences smoother if orga changes a question type.
                answer = Answer(
                    submission=self.submission,
                    person=self.speaker,
                    question=field.question,
                )
                self._save_to_answer(field, answer, v)
                answer.save()

    def _save_to_answer(self, field, answer, value):
        action = 'pretalx.submission.answer' + ('update' if answer.pk else 'create')
        if isinstance(field, forms.ModelMultipleChoiceField):
            answstr = ', '.join([str(o) for o in value])
            if not answer.pk:
                answer.save()
            else:
                answer.options.clear()
            answer.answer = answstr
            if value:
                answer.options.add(*value)
        elif isinstance(field, forms.ModelChoiceField):
            if not answer.pk:
                answer.save()
            else:
                answer.options.clear()
            if value:
                answer.options.add(value)
                answer.answer = value.answer
            else:
                answer.answer = ''
        elif isinstance(field, forms.FileField):
            if isinstance(value, UploadedFile):
                answer.answer_file.save(value.name, value)
                answer.answer = 'file://' + value.name
            value = answer.answer
        else:
            answer.answer = value
        answer.log_action(action, person=self.request_user, data={'answer': value})



from django.utils.translation import ugettext as _

from pretalx.common.phrases import Phrases


class OrgaPhrases(Phrases, app='orga'):

    logged_in = [
        _('Hi, nice to see you!'),
        _('Welcome!'),
        _('I hope you are having a good day :)'),
        _('Remember: organizing events is lots of work, but it pays off.'),
        _('If you are waiting for feedback from your speakers, try sending a mail to a subset of them.'),
        _('Remember to provide your speakers with all information they need ahead of time.'),
        _('Even the busiest event organizers should make time to see at least one talk ;)'),
    ]
    schedule_example_version = [
        'v1', 'v2', 'v4.0', 'v0.1', '♥',
    ]
    example_answer = [
        _('Pudding'), _('Panna cotta'), _('Chocolate chip cookies'),
    ]
    example_review = [
        _('I think this talk is well-suited to this conference, because ...'),
        _('I think this talk might fit the conference better, if ...'),
        _('I think this talk sounds like a perfect fit for Day 2, since ...'),
        _('I think this talk might be improved by adding ...'),
    ]
    another_review = [
        _('You\'re on a roll!'),
        _('"Just ONE more review, promise …"'),
        _('Review saved, thanks.'),
    ]
